/* This file was automatically generated, do not edit. Do not check in in version management. */

package org.harctoolbox.IrMaster;

import java.awt.Rectangle;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Properties;
import org.harctoolbox.harchardware.ir.GlobalCache;
import org.harctoolbox.harchardware.ir.IrTrans;
import org.harctoolbox.harchardware.ir.LircClient;

/**
 * This class handles the properties of the program, saved to a file between program invocations.
 */
public class Props {
    private final static boolean useXml = true;
    private static final long serialVersionUID = 1L;
    private Properties props;
    private String filename;
    private String applicationHome;
    private boolean needSave;
    private boolean wasReset = false;
    private final boolean isWindows = System.getProperty("os.name").startsWith("Windows");

    public interface IPropertyChangeListener {
        public void propertyChange(String name, Object oldValue, Object newValue);
    }

    private String ifWindows(String windows, String nonWindows) {
        return isWindows ? windows : nonWindows;
    }

    private HashMap<String,ArrayList<IPropertyChangeListener>> changeListeners;

   private void update(String key, String value) {
        if (!props.containsKey(key)) {
            if (value != null) {
                props.setProperty(key, value);
                needSave = true;
            }
        }
        changeListeners.put(key, new ArrayList<IPropertyChangeListener>());
    }

    public boolean getWasReset() {
        return wasReset;
    }

    private void setupDefaults() {
        update("bounds", null);
        update("plotBounds", null);
        update("disregardRepeatMins", "false");
        update("exportdir", System.getProperty("java.io.tmpdir") + File.separator + "exports");
        update("exportFormatFile", "exportformats_IrMaster.xml");
        update("globalcacheIpName", GlobalCache.defaultGlobalCacheIP);
        update("globalcacheModule", "2");
        update("globalcachePort", "1");
        update("globalcacheTimeOut", "2000");
        update("hardwareIndex", "0");
        update("helpfileUrl", (new File(applicationHome + File.separator + "doc" + File.separator + "IrMaster.html")).toURI().toString());
        update("irTransIpName", IrTrans.defaultIrTransIP);
        update("irTransPort", "0");
        update("irpmasterConfigfile", applicationHome + File.separator + "IrpProtocols.ini");
        update("irpmasterUrl", (new File(applicationHome + File.separator + "doc" + File.separator + "IrpMaster.html")).toURI().toString());
        update("lircIpName", LircClient.defaultLircIP);
        update("lircPort", Integer.toString(LircClient.lircDefaultPort));
        update("lookAndFeel", "0");
        update("makehexIrpdir", applicationHome + File.separator + "irps");
        update("outputFormat", "0");
        update("pingTimeout", "2000");
        update("protocol", "nec1");
        update("showEditMenu", "false");
        update("showExportPane", "false");
        update("showHardwarePane", "false");
        update("showIrp", "false");
        update("showRendererSelector", "false");
        update("showShortcutMenu", "false");
        update("showToolsMenu", "false");
        update("showWardialerPane", "false");
        update("usePopupsForErrors", "true");
        update("usePopupsForHelp", "true");

    }

    /**
     * Resets all properties to defaults.
     * This will probably leave the program in an inconsistent state,
     * so it should be restarted directly.
     */
    public void reset() {
        props = new Properties();
        changeListeners = new HashMap<String,ArrayList<IPropertyChangeListener>>();
        setupDefaults();
        needSave = true;
        wasReset = true;
    }

    /**
     * Sets up a Props instance from system default file name.
     * @param applicationHome
     */
    public Props(String applicationHome) {
        this(null, applicationHome);
    }

    /**
     * Sets up a Props instance from a given file name.
     * @param filename File to read from and, later, save to. Need not exist.
     * @param applicationHome
     */
    public Props(String filename, String applicationHome) {
        this.applicationHome = applicationHome;
        changeListeners = new HashMap<String,ArrayList<IPropertyChangeListener>>();
        this.filename = filename;
        if (filename == null || filename.isEmpty()) {
            String dir = System.getenv("LOCALAPPDATA"); // Win Vista and later
            if (dir == null) {
                dir = System.getenv("APPDATA"); // Win < Vista
            }
            if (dir != null) {
                dir = dir + File.separator + Version.appName;
                if (!(new File(dir)).isDirectory()) {
                    boolean status = (new File(dir)).mkdirs();
                    if (!status) {
                        System.err.println("Cannot create directory " + dir + ", using home directory instead.");
                    }
                }
            }
            this.filename = (dir != null)
                    ? (dir + File.separator + Version.appName + ".properties.xml")
                    : System.getProperty("user.home") + File.separator + "." + Version.appName + ".properties.xml";
        }

        needSave = false;
        props = new Properties();
        FileInputStream f = null;

        try {
            f = new FileInputStream(this.filename);
            if (useXml)
                props.loadFromXML(f);
            else
                props.load(f);
        } catch (FileNotFoundException ex) {
            System.err.println("Property File " + this.filename + " not found, using builtin defaults.");
            setupDefaults();
            needSave = true;
        } catch (IOException ex) {
            System.err.println("Property File " + this.filename + " could not be read, using builtin defaults.");
            setupDefaults();
            needSave = true;
        } finally {
            if (f != null) {
                try {
                    f.close();
                } catch (IOException ex) {
                    System.err.println(ex.getMessage());
                }
            }
        }
        setupDefaults();
    }

    /**
     * Save instance to given file name.
     *
     * @param filename Filename to be saved to.
     * @return success of operation
     * @throws IOException
     */
    public boolean save(File filename) throws IOException {
        if (!needSave && filename.getAbsolutePath().equals((new File(this.filename)).getAbsolutePath()))
            return false;

        FileOutputStream f;
        boolean success = false;
        try {
            f = new FileOutputStream(filename);
        } catch (FileNotFoundException ex) {
            throw (ex);
        }

        try {
            if (useXml)
                props.storeToXML(f, Version.appName + " properties, feel free to hand edit if desired");
            else
                props.store(f, Version.appName + " properties, feel free to hand edit if desired");

            success = true;
            needSave = false;
        } catch (IOException ex) {
            try {
                f.close();
            } catch (IOException exx) {
                System.err.println(exx.getMessage());
            }
            throw (ex);
        }

        try {
            f.close();
        } catch (IOException ex) {
            throw (ex);
        }

        return success;
    }

    /**
     * Saves the properties to the default, stored, file name.
     *
     * @return success of operation
     * @throws IOException
     */
    public String save() throws IOException {
        boolean result = save(new File(filename));
        return result ? filename : null;
    }

    // For debugging
    private void list() {
        props.list(System.err);
    }

    private void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
        ArrayList<IPropertyChangeListener> presentListeners = changeListeners.get(propertyName);
        for (IPropertyChangeListener listener : presentListeners)
            listener.propertyChange(propertyName, oldValue, newValue);
    }

    private void addPropertyChangeListener(String propertyName, IPropertyChangeListener listener) {
        ArrayList<IPropertyChangeListener> presentListeners = changeListeners.get(propertyName);
        if (!presentListeners.contains(listener))
            presentListeners.add(listener);
    }

    private void removePropertyChangeListener(String propertyName, IPropertyChangeListener listener) {
        ArrayList<IPropertyChangeListener> presentListeners = changeListeners.get(propertyName);
        if (presentListeners.contains(listener))
            presentListeners.remove(listener);
    }

/** @return Bounds of IrMaster window. */
    public Rectangle getBounds() {
        String str = props.getProperty("bounds");
        if (str == null || str.isEmpty())
            return null;
        String[] arr = str.trim().split(" +");
        return new Rectangle(Integer.parseInt(arr[0]), Integer.parseInt(arr[1]),
                Integer.parseInt(arr[2]), Integer.parseInt(arr[3]));
    }

    /** @param bounds Bounds of IrMaster window. */
    public void setBounds(Rectangle bounds) {
        String oldValue = props.getProperty("bounds");
        if (bounds == null)
            return;
        String newValue = String.format("%d %d %d %d", bounds.x, bounds.y, bounds.width, bounds.height);
        if (!newValue.equals(oldValue)) {
            props.setProperty("bounds", newValue);
            needSave = true;
            firePropertyChange("bounds", oldValue, newValue);
        }
    }

    public void addBoundsChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("bounds", listener);
    }

    public void removeBoundsChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("bounds", listener);
    }
    /** @return Bounds of IrMaster plot window. */
    public Rectangle getPlotBounds() {
        String str = props.getProperty("plotBounds");
        if (str == null || str.isEmpty())
            return null;
        String[] arr = str.trim().split(" +");
        return new Rectangle(Integer.parseInt(arr[0]), Integer.parseInt(arr[1]),
                Integer.parseInt(arr[2]), Integer.parseInt(arr[3]));
    }

    /** @param bounds Bounds of IrMaster plot window. */
    public void setPlotBounds(Rectangle bounds) {
        String oldValue = props.getProperty("plotBounds");
        if (bounds == null)
            return;
        String newValue = String.format("%d %d %d %d", bounds.x, bounds.y, bounds.width, bounds.height);
        if (!newValue.equals(oldValue)) {
            props.setProperty("plotBounds", newValue);
            needSave = true;
            firePropertyChange("plotBounds", oldValue, newValue);
        }
    }

    public void addPlotBoundsChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("plotBounds", listener);
    }

    public void removePlotBoundsChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("plotBounds", listener);
    }
    /** @return Value of the "disregard repeat mins" parameter. See IrpMaster documentation for the semantic. */
    public boolean getDisregardRepeatMins() {
        return Boolean.parseBoolean(props.getProperty("disregardRepeatMins"));
    }

    /** @param val Value of the "disregard repeat mins" parameter. See IrpMaster documentation for the semantic. */
    public void setDisregardRepeatMins(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("disregardRepeatMins"));
        if (oldValue != val) {
            props.setProperty("disregardRepeatMins", Boolean.toString(val));
            needSave = true;
            firePropertyChange("disregardRepeatMins", oldValue, val);
        }
    }

    public void addDisregardRepeatMinsChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("disregardRepeatMins", listener);
    }

    public void removeDisregardRepeatMinsChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("disregardRepeatMins", listener);
    }
    /** @return Directory to which to write exports. */
    public String getExportdir() {
        return props.getProperty("exportdir");
    }

    /** @param str Directory to which to write exports. */
    public void setExportdir(String str) {
        String oldValue = props.getProperty("exportdir");
        if (!oldValue.equals(str)) {
            props.setProperty("exportdir", str);
            firePropertyChange("exportdir", oldValue, str);
            needSave = true;
        }
    }

    public void addExportdirChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("exportdir", listener);
    }

    public void removeExportdirChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("exportdir", listener);
    }
    /** @return Filename of a file consisting of export format descriptions. */
    public String getExportFormatFile() {
        return props.getProperty("exportFormatFile");
    }

    /** @param str Filename of a file consisting of export format descriptions. */
    public void setExportFormatFile(String str) {
        String oldValue = props.getProperty("exportFormatFile");
        if (!oldValue.equals(str)) {
            props.setProperty("exportFormatFile", str);
            firePropertyChange("exportFormatFile", oldValue, str);
            needSave = true;
        }
    }

    public void addExportFormatFileChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("exportFormatFile", listener);
    }

    public void removeExportFormatFileChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("exportFormatFile", listener);
    }
    /** @return IP Name or Address of GlobalCache to use. */
    public String getGlobalcacheIpName() {
        return props.getProperty("globalcacheIpName");
    }

    /** @param str IP Name or Address of GlobalCache to use. */
    public void setGlobalcacheIpName(String str) {
        String oldValue = props.getProperty("globalcacheIpName");
        if (!oldValue.equals(str)) {
            props.setProperty("globalcacheIpName", str);
            firePropertyChange("globalcacheIpName", oldValue, str);
            needSave = true;
        }
    }

    public void addGlobalcacheIpNameChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("globalcacheIpName", listener);
    }

    public void removeGlobalcacheIpNameChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("globalcacheIpName", listener);
    }
    /** @return Module number of Global Cache to use, see its documentation. */
    public int getGlobalcacheModule() {
        return Integer.parseInt(props.getProperty("globalcacheModule"));
    }

    /** @param n Module number of Global Cache to use, see its documentation. */
    public void setGlobalcacheModule(int n) {
        int oldValue = Integer.parseInt(props.getProperty("globalcacheModule"));
        if (oldValue != n) {
            props.setProperty("globalcacheModule", Integer.toString(n));
            needSave = true;
            firePropertyChange("globalcacheModule", oldValue, n);
        }
    }

    public void addGlobalcacheModuleChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("globalcacheModule", listener);
    }

    public void removeGlobalcacheModuleChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("globalcacheModule", listener);
    }
    /** @return IR Port number of selected Global Cache module, see Global Cache documenation. */
    public int getGlobalcachePort() {
        return Integer.parseInt(props.getProperty("globalcachePort"));
    }

    /** @param n IR Port number of selected Global Cache module, see Global Cache documenation. */
    public void setGlobalcachePort(int n) {
        int oldValue = Integer.parseInt(props.getProperty("globalcachePort"));
        if (oldValue != n) {
            props.setProperty("globalcachePort", Integer.toString(n));
            needSave = true;
            firePropertyChange("globalcachePort", oldValue, n);
        }
    }

    public void addGlobalcachePortChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("globalcachePort", listener);
    }

    public void removeGlobalcachePortChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("globalcachePort", listener);
    }
    /** @return Socket timeout for Global Cache */
    public int getGlobalcacheTimeOut() {
        return Integer.parseInt(props.getProperty("globalcacheTimeOut"));
    }

    /** @param n Socket timeout for Global Cache */
    public void setGlobalcacheTimeOut(int n) {
        int oldValue = Integer.parseInt(props.getProperty("globalcacheTimeOut"));
        if (oldValue != n) {
            props.setProperty("globalcacheTimeOut", Integer.toString(n));
            needSave = true;
            firePropertyChange("globalcacheTimeOut", oldValue, n);
        }
    }

    public void addGlobalcacheTimeOutChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("globalcacheTimeOut", listener);
    }

    public void removeGlobalcacheTimeOutChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("globalcacheTimeOut", listener);
    }
    /** @return System dependent semantics. */
    public int getHardwareIndex() {
        return Integer.parseInt(props.getProperty("hardwareIndex"));
    }

    /** @param n System dependent semantics. */
    public void setHardwareIndex(int n) {
        int oldValue = Integer.parseInt(props.getProperty("hardwareIndex"));
        if (oldValue != n) {
            props.setProperty("hardwareIndex", Integer.toString(n));
            needSave = true;
            firePropertyChange("hardwareIndex", oldValue, n);
        }
    }

    public void addHardwareIndexChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("hardwareIndex", listener);
    }

    public void removeHardwareIndexChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("hardwareIndex", listener);
    }
    /** @return URL for help file. */
    public String getHelpfileUrl() {
        return props.getProperty("helpfileUrl");
    }

    /** @param str URL for help file. */
    public void setHelpfileUrl(String str) {
        String oldValue = props.getProperty("helpfileUrl");
        if (!oldValue.equals(str)) {
            props.setProperty("helpfileUrl", str);
            firePropertyChange("helpfileUrl", oldValue, str);
            needSave = true;
        }
    }

    public void addHelpfileUrlChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("helpfileUrl", listener);
    }

    public void removeHelpfileUrlChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("helpfileUrl", listener);
    }
    /** @return IP name or address of IRTrans unit. */
    public String getIrTransIpName() {
        return props.getProperty("irTransIpName");
    }

    /** @param str IP name or address of IRTrans unit. */
    public void setIrTransIpName(String str) {
        String oldValue = props.getProperty("irTransIpName");
        if (!oldValue.equals(str)) {
            props.setProperty("irTransIpName", str);
            firePropertyChange("irTransIpName", oldValue, str);
            needSave = true;
        }
    }

    public void addIrTransIpNameChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("irTransIpName", listener);
    }

    public void removeIrTransIpNameChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("irTransIpName", listener);
    }
    /** @return IRTrans port to use. */
    public int getIrTransPort() {
        return Integer.parseInt(props.getProperty("irTransPort"));
    }

    /** @param n IRTrans port to use. */
    public void setIrTransPort(int n) {
        int oldValue = Integer.parseInt(props.getProperty("irTransPort"));
        if (oldValue != n) {
            props.setProperty("irTransPort", Integer.toString(n));
            needSave = true;
            firePropertyChange("irTransPort", oldValue, n);
        }
    }

    public void addIrTransPortChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("irTransPort", listener);
    }

    public void removeIrTransPortChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("irTransPort", listener);
    }
    /** @return Filename of IrpProtocols.ini */
    public String getIrpmasterConfigfile() {
        return props.getProperty("irpmasterConfigfile");
    }

    /** @param str Filename of IrpProtocols.ini */
    public void setIrpmasterConfigfile(String str) {
        String oldValue = props.getProperty("irpmasterConfigfile");
        if (!oldValue.equals(str)) {
            props.setProperty("irpmasterConfigfile", str);
            firePropertyChange("irpmasterConfigfile", oldValue, str);
            needSave = true;
        }
    }

    public void addIrpmasterConfigfileChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("irpmasterConfigfile", listener);
    }

    public void removeIrpmasterConfigfileChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("irpmasterConfigfile", listener);
    }
    /** @return URL of IrpMaster documentation. */
    public String getIrpmasterUrl() {
        return props.getProperty("irpmasterUrl");
    }

    /** @param str URL of IrpMaster documentation. */
    public void setIrpmasterUrl(String str) {
        String oldValue = props.getProperty("irpmasterUrl");
        if (!oldValue.equals(str)) {
            props.setProperty("irpmasterUrl", str);
            firePropertyChange("irpmasterUrl", oldValue, str);
            needSave = true;
        }
    }

    public void addIrpmasterUrlChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("irpmasterUrl", listener);
    }

    public void removeIrpmasterUrlChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("irpmasterUrl", listener);
    }
    /** @return IP name or address of LIRC server. */
    public String getLircIpName() {
        return props.getProperty("lircIpName");
    }

    /** @param str IP name or address of LIRC server. */
    public void setLircIpName(String str) {
        String oldValue = props.getProperty("lircIpName");
        if (!oldValue.equals(str)) {
            props.setProperty("lircIpName", str);
            firePropertyChange("lircIpName", oldValue, str);
            needSave = true;
        }
    }

    public void addLircIpNameChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("lircIpName", listener);
    }

    public void removeLircIpNameChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("lircIpName", listener);
    }
    /** @return TCP port number of LIRC sserver, typically 8765. */
    public int getLircPort() {
        return Integer.parseInt(props.getProperty("lircPort"));
    }

    /** @param n TCP port number of LIRC sserver, typically 8765. */
    public void setLircPort(int n) {
        int oldValue = Integer.parseInt(props.getProperty("lircPort"));
        if (oldValue != n) {
            props.setProperty("lircPort", Integer.toString(n));
            needSave = true;
            firePropertyChange("lircPort", oldValue, n);
        }
    }

    public void addLircPortChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("lircPort", listener);
    }

    public void removeLircPortChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("lircPort", listener);
    }
    /** @return Look and feel, as integer index in table. Semantics is system dependent. */
    public int getLookAndFeel() {
        return Integer.parseInt(props.getProperty("lookAndFeel"));
    }

    /** @param n Look and feel, as integer index in table. Semantics is system dependent. */
    public void setLookAndFeel(int n) {
        int oldValue = Integer.parseInt(props.getProperty("lookAndFeel"));
        if (oldValue != n) {
            props.setProperty("lookAndFeel", Integer.toString(n));
            needSave = true;
            firePropertyChange("lookAndFeel", oldValue, n);
        }
    }

    public void addLookAndFeelChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("lookAndFeel", listener);
    }

    public void removeLookAndFeelChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("lookAndFeel", listener);
    }
    /** @return Directory of Makehex IRP files. */
    public String getMakehexIrpdir() {
        return props.getProperty("makehexIrpdir");
    }

    /** @param str Directory of Makehex IRP files. */
    public void setMakehexIrpdir(String str) {
        String oldValue = props.getProperty("makehexIrpdir");
        if (!oldValue.equals(str)) {
            props.setProperty("makehexIrpdir", str);
            firePropertyChange("makehexIrpdir", oldValue, str);
            needSave = true;
        }
    }

    public void addMakehexIrpdirChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("makehexIrpdir", listener);
    }

    public void removeMakehexIrpdirChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("makehexIrpdir", listener);
    }
    /** @return System dependant syntax. */
    public int getOutputFormat() {
        return Integer.parseInt(props.getProperty("outputFormat"));
    }

    /** @param n System dependant syntax. */
    public void setOutputFormat(int n) {
        int oldValue = Integer.parseInt(props.getProperty("outputFormat"));
        if (oldValue != n) {
            props.setProperty("outputFormat", Integer.toString(n));
            needSave = true;
            firePropertyChange("outputFormat", oldValue, n);
        }
    }

    public void addOutputFormatChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("outputFormat", listener);
    }

    public void removeOutputFormatChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("outputFormat", listener);
    }
    /** @return Ping timeout in milli seconds. */
    public int getPingTimeout() {
        return Integer.parseInt(props.getProperty("pingTimeout"));
    }

    /** @param n Ping timeout in milli seconds. */
    public void setPingTimeout(int n) {
        int oldValue = Integer.parseInt(props.getProperty("pingTimeout"));
        if (oldValue != n) {
            props.setProperty("pingTimeout", Integer.toString(n));
            needSave = true;
            firePropertyChange("pingTimeout", oldValue, n);
        }
    }

    public void addPingTimeoutChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("pingTimeout", listener);
    }

    public void removePingTimeoutChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("pingTimeout", listener);
    }
    /** @return Protocol name, as in IrpProtocols.ini */
    public String getProtocol() {
        return props.getProperty("protocol");
    }

    /** @param str Protocol name, as in IrpProtocols.ini */
    public void setProtocol(String str) {
        String oldValue = props.getProperty("protocol");
        if (!oldValue.equals(str)) {
            props.setProperty("protocol", str);
            firePropertyChange("protocol", oldValue, str);
            needSave = true;
        }
    }

    public void addProtocolChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("protocol", listener);
    }

    public void removeProtocolChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("protocol", listener);
    }
    
    public boolean getShowEditMenu() {
        return Boolean.parseBoolean(props.getProperty("showEditMenu"));
    }

    
    public void setShowEditMenu(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("showEditMenu"));
        if (oldValue != val) {
            props.setProperty("showEditMenu", Boolean.toString(val));
            needSave = true;
            firePropertyChange("showEditMenu", oldValue, val);
        }
    }

    public void addShowEditMenuChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("showEditMenu", listener);
    }

    public void removeShowEditMenuChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("showEditMenu", listener);
    }
    
    public boolean getShowExportPane() {
        return Boolean.parseBoolean(props.getProperty("showExportPane"));
    }

    
    public void setShowExportPane(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("showExportPane"));
        if (oldValue != val) {
            props.setProperty("showExportPane", Boolean.toString(val));
            needSave = true;
            firePropertyChange("showExportPane", oldValue, val);
        }
    }

    public void addShowExportPaneChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("showExportPane", listener);
    }

    public void removeShowExportPaneChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("showExportPane", listener);
    }
    
    public boolean getShowHardwarePane() {
        return Boolean.parseBoolean(props.getProperty("showHardwarePane"));
    }

    
    public void setShowHardwarePane(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("showHardwarePane"));
        if (oldValue != val) {
            props.setProperty("showHardwarePane", Boolean.toString(val));
            needSave = true;
            firePropertyChange("showHardwarePane", oldValue, val);
        }
    }

    public void addShowHardwarePaneChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("showHardwarePane", listener);
    }

    public void removeShowHardwarePaneChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("showHardwarePane", listener);
    }
    
    public boolean getShowIrp() {
        return Boolean.parseBoolean(props.getProperty("showIrp"));
    }

    
    public void setShowIrp(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("showIrp"));
        if (oldValue != val) {
            props.setProperty("showIrp", Boolean.toString(val));
            needSave = true;
            firePropertyChange("showIrp", oldValue, val);
        }
    }

    public void addShowIrpChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("showIrp", listener);
    }

    public void removeShowIrpChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("showIrp", listener);
    }
    
    public boolean getShowRendererSelector() {
        return Boolean.parseBoolean(props.getProperty("showRendererSelector"));
    }

    
    public void setShowRendererSelector(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("showRendererSelector"));
        if (oldValue != val) {
            props.setProperty("showRendererSelector", Boolean.toString(val));
            needSave = true;
            firePropertyChange("showRendererSelector", oldValue, val);
        }
    }

    public void addShowRendererSelectorChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("showRendererSelector", listener);
    }

    public void removeShowRendererSelectorChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("showRendererSelector", listener);
    }
    
    public boolean getShowShortcutMenu() {
        return Boolean.parseBoolean(props.getProperty("showShortcutMenu"));
    }

    
    public void setShowShortcutMenu(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("showShortcutMenu"));
        if (oldValue != val) {
            props.setProperty("showShortcutMenu", Boolean.toString(val));
            needSave = true;
            firePropertyChange("showShortcutMenu", oldValue, val);
        }
    }

    public void addShowShortcutMenuChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("showShortcutMenu", listener);
    }

    public void removeShowShortcutMenuChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("showShortcutMenu", listener);
    }
    
    public boolean getShowToolsMenu() {
        return Boolean.parseBoolean(props.getProperty("showToolsMenu"));
    }

    
    public void setShowToolsMenu(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("showToolsMenu"));
        if (oldValue != val) {
            props.setProperty("showToolsMenu", Boolean.toString(val));
            needSave = true;
            firePropertyChange("showToolsMenu", oldValue, val);
        }
    }

    public void addShowToolsMenuChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("showToolsMenu", listener);
    }

    public void removeShowToolsMenuChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("showToolsMenu", listener);
    }
    
    public boolean getShowWardialerPane() {
        return Boolean.parseBoolean(props.getProperty("showWardialerPane"));
    }

    
    public void setShowWardialerPane(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("showWardialerPane"));
        if (oldValue != val) {
            props.setProperty("showWardialerPane", Boolean.toString(val));
            needSave = true;
            firePropertyChange("showWardialerPane", oldValue, val);
        }
    }

    public void addShowWardialerPaneChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("showWardialerPane", listener);
    }

    public void removeShowWardialerPaneChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("showWardialerPane", listener);
    }
    /** @return If true, use popups for help. Otherwise the console will be used. */
    public boolean getUsePopupsForErrors() {
        return Boolean.parseBoolean(props.getProperty("usePopupsForErrors"));
    }

    /** @param val If true, use popups for help. Otherwise the console will be used. */
    public void setUsePopupsForErrors(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("usePopupsForErrors"));
        if (oldValue != val) {
            props.setProperty("usePopupsForErrors", Boolean.toString(val));
            needSave = true;
            firePropertyChange("usePopupsForErrors", oldValue, val);
        }
    }

    public void addUsePopupsForErrorsChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("usePopupsForErrors", listener);
    }

    public void removeUsePopupsForErrorsChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("usePopupsForErrors", listener);
    }
    /** @return If true, use popups for help. Otherwise the console will be used. */
    public boolean getUsePopupsForHelp() {
        return Boolean.parseBoolean(props.getProperty("usePopupsForHelp"));
    }

    /** @param val If true, use popups for help. Otherwise the console will be used. */
    public void setUsePopupsForHelp(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("usePopupsForHelp"));
        if (oldValue != val) {
            props.setProperty("usePopupsForHelp", Boolean.toString(val));
            needSave = true;
            firePropertyChange("usePopupsForHelp", oldValue, val);
        }
    }

    public void addUsePopupsForHelpChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("usePopupsForHelp", listener);
    }

    public void removeUsePopupsForHelpChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("usePopupsForHelp", listener);
    }
    
    /**
     * Main routine for testing and debugging.
     * @param args filename
     */
    public static void main(String[] args) {
        String filename = args.length > 0 ? args[0] : null;
        try {
            Props p = new Props(filename);
            p.list();
            p.save();
        } catch (IOException e) {
            System.err.println(e.getMessage());
        }
    }
}
